/*
 * $Revision: 1.8 $ $Date: 2005/12/17 09:30:09 $
 * (C) 2004 SUGIMOTO Ken-ichi
 * 쐬F 2004/08/27
 */

package feat.v1.impl.template;

import java.util.List;

import org.apache.commons.jxpath.JXPathContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import feat.v1.StringUtil;
import feat.v1.template.HTMLAttribute;
import feat.v1.template.HTMLDocument;
import feat.v1.template.HTMLElement;
import feat.v1.template.HTMLNode;
import feat.v1.template.HTMLNodeList;
import feat.v1.template.NodeIterator;
import feat.v1.template.NodeListIterator;
import feat.v1.template.NodeNotFoundException;

/**
 * HTML̃^O\NXB
 * JnEI^ÕyAAJn^Ô݁AI^Ô݂̏ꍇB
 */
public class HTMLElementImpl extends HTMLCompositeNodeImpl implements HTMLElement, HTMLDocument {

    private static Log log = LogFactory.getLog(HTMLElementImpl.class);

    private HTMLAttribute firstAttribute;
    private String tagName;
    private String id;
    private String[] classList;
    private String encoding;
    private boolean startTag;
    private boolean endTag;

    HTMLElementImpl(boolean isDocumentNode) {
        this(null, true, true, isDocumentNode);
    }

    HTMLElementImpl(String tagName, boolean startTag, boolean endTag, boolean isDocumentNode) {
        this.tagName = tagName;
        this.startTag = startTag;
        this.endTag = endTag;

        if ( isDocumentNode )
            setDomNode(new DOMDocumentImpl(this));
        else
            setDomNode(new DOMElementImpl(this));
    }

    public boolean isDocumentNode() {
        return getDomNode() instanceof Document;
    }

    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

    /*void addAttribute(HTMLAttribute attr) {
        addAttribute(attr, true);
    }*/

    /**
     * ǉB
     * @param attr P̂̑m[hBXg\̃m[h͐󂯓ȂB
     * @param withWhitespace attȓOɃXy[XȂtrue
     */
    void addAttribute(HTMLAttribute attr, boolean withWhitespace) {
        if ( attr == null )
            return;

        // Ō̑m[h擾
        // m[hȂattr𑮐m[hɐݒ肵ă^[
        HTMLAttribute last = firstAttribute != null ? (HTMLAttribute)firstAttribute.getLast() : null;
        if ( last == null ) {
            firstAttribute = attr;
            ((HTMLNodeImpl)attr).setParent(this);
            return;
        }

        // lt
        if ( attr instanceof HTMLValuedAttribute ) {
            // Ōオ󔒂ł͂ȂXy[Xǉ
            if ( withWhitespace && !(last instanceof HTMLWhitespaceAttribute) ) {
                HTMLWhitespaceAttribute spaceNode = new HTMLWhitespaceAttribute(" ");
                last.insertAfter(spaceNode);
                last = spaceNode;
            }
            last.insertAfter(attr);

            HTMLValuedAttribute vattr = (HTMLValuedAttribute)attr;
            String name = vattr.getName();
            if ( name != null ) {
                name = name.toLowerCase();
                if ( name.equals("id") ) {
                    this.id = vattr.getValue();
                }
                else if ( name.equals("class") ) {
                    setClassList(vattr.getValue());
                }
            }
        }
        // Xy[X
        else if ( attr instanceof HTMLWhitespaceAttribute ) {
            // ŌオXy[X當ǉ
            if ( last instanceof HTMLWhitespaceAttribute ) {
                ((HTMLWhitespaceAttribute)last).addText(attr.getValue());
            }
            else {
                last.insertAfter(attr);
            }
        }
        else
            last.insertAfter(attr);
    }

    /*void addWhitespace(String str) {
        if ( str == null || str.length() == 0 ) {
            str = " ";
        }

        HTMLAttribute attr = (HTMLAttribute)NodeSelectUtil.getLastNode(firstAttribute);
        if ( attr != null && attr instanceof HTMLWhitespaceAttribute ) {
            ((HTMLWhitespaceAttribute)attr).addText(str);
        }
        else {
            NodeControlUtil.insertAfter(attr, new HTMLWhitespaceAttribute(str));
        }
    }*/

    public boolean hasStartTag() {
        return startTag;
    }

    /**
     * Xg̍ŏ̃Xy[XׂĂ̗vfЂƂ̑ɂ܂Ƃ߂B
     */
    void assemble() {
        if ( firstAttribute != null ) {
            // ׂč폜ēƗ̃Xg𓾂
            HTMLAttribute attrList = (HTMLAttribute)detachAll(firstAttribute);
            firstAttribute = null;

            // Xg̍ŏ̃m[hXy[XȂvf̑ɒǉ
            if ( attrList instanceof HTMLWhitespaceAttribute ) {
                HTMLNode wsNode = attrList;
                attrList = (HTMLAttribute)((HTMLNodeImpl)attrList).next;
                wsNode.detach();
                if ( attrList == null )
                    return;
            }

            // XgɃm[hcĂ炷ׂĂ1̑ɂ܂Ƃ߂
            StringBuffer buf = TemplateUtil.nodeListToHTML(attrList, new StringBuffer());
            HTMLValuedAttribute a = new HTMLValuedAttribute(null, buf.toString());
            a.setQuote(null);
            addAttribute(a, false);
        }
    }

    HTMLNodeMapImpl getAttributes() {
        HTMLNodeMapImpl ret = new HTMLNodeMapImpl();
        for(NodeIterator it=new NodeListIterator(firstAttribute); it.hasNext(); ) {
            HTMLNode node = it.nextNode();
            if ( node instanceof HTMLValuedAttribute ) {
                HTMLValuedAttribute attr = (HTMLValuedAttribute)node;
                if ( attr.getName() != null )
                    ret.put(attr.getName().toLowerCase(), attr);
            }
        }
        return ret;
    }

    public String toString() {
        return startTagToString(new StringBuffer()).toString();
    }

    // HTMLDocument̎ -------------------------------------------------------

    public String getEncoding() {
        return encoding;
    }

    public HTMLNode find(String xpath) throws NodeNotFoundException {
        Node domNode = getDomNode();

        JXPathContext context = JXPathContext.newContext(domNode);
        context.setLenient(true);
        Node found = (Node)context.selectSingleNode(xpath);
        HTMLNode ret = null;
        if ( found instanceof HTMLConvertibleNode ) {
            ret = ((HTMLConvertibleNode)found).convertHTML();
        }

        if ( ret == null )
            throw new NodeNotFoundException("xpath:"+xpath);

        return ret;
    }

    public HTMLNodeList select(String xpath) {
        Node domNode = getDomNode();
        if ( domNode == null )
            return null;

        JXPathContext context = JXPathContext.newContext(domNode);
        context.setLenient(true);
        List list = context.selectNodes(xpath);

        HTMLNodeListImpl ret = new HTMLNodeListImpl();
        for(int i=0; i<list.size(); i++) {
            ret.add(((HTMLConvertibleNode)list.get(i)).convertHTML());
        }
        return ret;
    }

    public String toHTML() {
        StringBuffer buf = new StringBuffer();
        toHTML(buf);
        return buf.toString();
    }


    // HTMLElement̎ --------------------------------------------------------

    public String getAttribute(String name) {
        String nameL = name.toLowerCase();
        if ( nameL.equals("id") ) {
            return this.id;
        }
        else {
            HTMLValuedAttribute attr = getValuedAttribute(name);
            if ( attr == null )
                return null;
            else
                return attr.getValue();
        }
    }

    public boolean containsClassValue(String classValue) {
        if ( classList != null )
            for(int i=0; i<classList.length; i++) {
                if ( classList[i].equals(classValue) )
                    return true;
            }
        return false;
    }

    /**
     * lt̑ԂB
     * @param name
     * @return Ȃnull
     */
    HTMLValuedAttribute getValuedAttribute(String name) {
        NodeIterator it = new NodeListIterator(firstAttribute);
        while(it.hasNext()) {
            Object a = it.next();
            if ( a instanceof HTMLValuedAttribute ) {
                HTMLValuedAttribute attr = (HTMLValuedAttribute)a;
                if ( attr.getName().equalsIgnoreCase(name) ) {
                    return attr;
                }
            }
        }
        return null;
    }

    /**
     * vf̑ݒ肷B͑啶ʂȂB
     * ̓GXP[vB
     * @param name String
     * @param value String
     */
    public void setAttribute(String name, String value) {
        String nameL = name.toLowerCase();
        if ( nameL.equals("id") ) {
            this.id = value;
        }
        else if ( nameL.equals("class") ) {
            setClassList(value);
        }

        HTMLValuedAttribute attr = getValuedAttribute(name);
        if ( attr == null ) {
            attr = new HTMLValuedAttribute(name, null);
            addAttribute(attr, true);
        }
        attr.setValue(value);
    }

    private void setClassList(String classValue) {
        classList = StringUtil.split(classValue);
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        setAttribute("id", id);
    }

    public String[] getClassList() {
        return classList;
    }

    public void setClassValue(String classValue) {
        setAttribute("class", classValue);
    }

    public boolean hasAttribute() {
        NodeIterator it = new NodeListIterator(firstAttribute);
        while(it.hasNext()) {
            Object a = it.next();
            if ( a instanceof HTMLValuedAttribute ) {
                return true;
            }
        }
        return false;
    }

    public boolean hasAttribute(String name) {
        HTMLValuedAttribute attr = getValuedAttribute(name);
        return attr != null;
    }

    public void removeAttribute(String name) {
        String nameL = name.toLowerCase();
        if ( nameL.equals("id") ) {
            this.id = null;
        }
        else if ( nameL.equals("class") ) {
            this.classList = null;
        }

        HTMLValuedAttribute attr = getValuedAttribute(name);
        attr.setParent(null);
        if ( attr == firstAttribute ) {
            firstAttribute = (HTMLAttribute)attr.next;
        }
        attr.detach();
    }

    public String getTagName() {
        return tagName;
    }

    /**
     * ^Oݒ肷B
     * nullw肷ƃ^O񂪏o͂ȂȂB
     * @param name String
     */
    public void setTagName(String name) {
        tagName = name;
    }

    public boolean hasEndTag() {
        return endTag;
    }

    /**
     * I^OKvȂtrueZbgB
     * @param v boolean
     */
    public void setEndTag(boolean v) {
        endTag = v;
    }


    // HTMLNode̎ -----------------------------------------------------------

    public String getValue() {
        return null;
    }

    public HTMLNode copy(boolean deep) {
        HTMLElementImpl ret = new HTMLElementImpl(isDocumentNode());

        // vpeB̃Rs[
        ret.tagName = tagName;
        ret.id = id;
        if ( classList != null )
            ret.classList = (String[])classList.clone();
        ret.startTag = startTag;
        ret.endTag = endTag;
        ret.encoding = encoding;

        // ̃Rs[
        if ( firstAttribute != null ) {
            ret.firstAttribute = (HTMLAttribute)TemplateUtil.copyNodeList((HTMLNodeImpl)firstAttribute, ret);
        }

        if ( deep ) {
            ret.setFirstChild(TemplateUtil.copyNodeList(firstChild, ret));
        }

        return ret;
    }

    public StringBuffer toHTML(StringBuffer buf) {
        // Jn^O
        startTagToString(buf);

        // qm[h
        TemplateUtil.nodeListToHTML(firstChild, buf);

        // I^O
        endTagToString(buf);

        return buf;
    }

    private StringBuffer startTagToString(StringBuffer buf) {
        if ( tagName != null && startTag ) {
            buf.append("<").append(tagName);
            TemplateUtil.nodeListToHTML(firstAttribute, buf);
            buf.append(">");
        }
        return buf;
    }

    private StringBuffer endTagToString(StringBuffer buf) {
        if ( (endTag || firstChild != null) && tagName != null ) {
            buf.append("</").append(tagName).append(">");
        }
        return buf;
    }

    public int getNodeType() {
        return isDocumentNode() ? NODETYPE_DOCUMENT : NODETYPE_ELEMENT;
    }
}
